"""
Utilities for handling date and time
"""
from datetime import datetime, timezone
from pytz import UTC
from tzlocal import get_localzone


def utc_now():
    """
    Return an aware :py:class:`datetime.datetime` instance of the current
    date and time, in UTC timezone

    :return: Current date and time, in UTC timezone
    :rtype: datetime.datetime
    """
    return datetime.now(timezone.utc)


def is_aware(
        time_point):
    """
    Return whether ``time_point`` is aware of the timezone
    """
    return time_point.tzinfo is not None and \
        time_point.utcoffset() is not None


def make_aware(
        time_point):
    """
    Return an aware time point

    :param datetime.datetime time_point: Unaware time point in UTC
    :return: Aware time point in UTC timezone
    :rtype: datetime.datetime
    """
    assert not is_aware(time_point)

    return time_point.replace(tzinfo=UTC)


def local_time_point(
        time_point):
    """
    Return a time point with the same UTC time as ``time_point``, but
    in the local time zone

    :param datetime.datetime time_point: Aware time point
    :return: Time point in local timezone
    :rtype: datetime.datetime
    """
    assert is_aware(time_point)

    return time_point.astimezone(get_localzone())



from flask import jsonify
from werkzeug.exceptions import HTTPException


def response(
        code,
        description):
    """
    Format a response

    :param int code: HTTP error code
    :param str description: Error message
    :return: Tuple of a wrapped JSON snippet and the error code
    :rtype: Tuple of :py:class:`flask.Response` containing a JSON snippet,
        and the error code

    The JSON snippet is formatted like this:

    .. code-block:: json

       {
           "status_code": 404,
           "message": "The requested URL was not found on the server"
       }
    """

    payload = jsonify({
        "status_code": code,
        "message": description
    })
    print(type(payload))

    return payload, code


def http_exception_error_handler(
        exception):
    """
    Handle HTTP exception

    :param werkzeug.exceptions.HTTPException exception: Raised exception

    A response is returned, as formatted by the :py:func:`response` function.
    """

    assert issubclass(type(exception), HTTPException), type(exception)
    assert hasattr(exception, "code")
    assert hasattr(exception, "description")

    return response(exception.code, exception.description)



"""
Error handling
"""
from . import error_code


def register(
        app):
    """
    Register all available error handlers

    :param flask.Flask app: Application instance
    """
    error_code.register(app)



"""
HTTP error code error handling
"""
from . import json


def register(
        app):
    """
    Register all HTTP error code error handlers

    Currently, errors are handled by the JSON error handler.
    """

    # Pick a handler based on the requested format. Currently we assume the
    # caller wants JSON.
    error_handler = json.http_exception_error_handler


    @app.errorhandler(400)
    def handle_bad_request(
            exception):
        return error_handler(exception)


    @app.errorhandler(404)
    def handle_not_found(
            exception):
        return error_handler(exception)


    @app.errorhandler(405)
    def handle_method_not_allowed(
            exception):
        return error_handler(exception)


    @app.errorhandler(422)
    def handle_unprocessable_entity(
            exception):
        return error_handler(exception)


    @app.errorhandler(500)
    def handle_internal_server_error(
            exception):
        return error_handler(exception)



import json
import requests


class Catalog(object):

    def __init__(self,
            uri,
            username,
            password):
        self.uri = uri
        self.auth = (username, password)


    def workspaces(self):

        headers = {
            "Accept": "application/json"
        }
        response = requests.get(
            "{}/workspaces".format(self.uri),
            headers=headers,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)

        data = response.json()
        workspaces = data["workspaces"]["workspace"]

        return workspaces


    def workspace(self,
            name):

        headers = {
            "Accept": "application/json"
        }
        response = requests.get(
            "{}/workspaces/{}".format(self.uri, name),
            headers=headers,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)

        data = response.json()
        workspace = data["workspace"]

        return workspace


    def workspace_exists(self,
            name):

        parameters = {
            "quietOnNotFound": "true"
        }
        response = requests.get(
            "{}/workspaces/{}".format(self.uri, name),
            params=parameters,
            auth=self.auth)

        return response.status_code == 200


    def create_workspace(self,
            name):

        headers = {
            "Content-type": "application/json"
        }
        payload = {
            "workspace": {
                "name": name
            }
        }
        response = requests.post(
            "{}/workspaces".format(self.uri),
            headers=headers,
            data=json.dumps(payload),
            auth=self.auth)

        if response.status_code != 201:
            raise RuntimeError(response.text)


    def delete_workspace(self,
            name,
            recurse=False):

        headers = {
            "Content-type": "application/json"
        }
        parameters = {
            "recurse": "true" if recurse else "false"
        }
        response = requests.delete(
            "{}/workspaces/{}".format(self.uri, name),
            headers=headers,
            params=parameters,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)


    def coverage_stores(self,
            workspace_name):

        headers = {
            "Accept": "application/json"
        }
        response = requests.get(
            "{}/workspaces/{}/coveragestores".format(self.uri, workspace_name),
            headers=headers,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)


        data = response.json()
        coverage_stores = data["coverageStores"]["coverageStore"]

        return coverage_stores


    def coverage_store(self,
            workspace_name,
            name):

        headers = {
            "Accept": "application/json"
        }
        response = requests.get(
            "{}/workspaces/{}/coveragestores/{}".format(self.uri,
                workspace_name, name),
            headers=headers,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)

        data = response.json()
        coverage_store = data["coverageStore"]

        return coverage_store


    def coverage_store_exists(self,
            workspace_name,
            name):

        parameters = {
            "quietOnNotFound": "true"
        }
        response = requests.get(
            "{}/workspaces/{}/coveragestores/{}".format(self.uri,
                workspace_name, name),
            params=parameters,
            auth=self.auth)

        return response.status_code == 200


    def create_coverage_store(self,
            workspace_name,
            name,
            type,
            pathname):

        headers = {
            "Content-type": "application/json"
        }
        payload = {
            "coverageStore": {
                "name": name,
                "type": type,
                "enabled": "true",
                "workspace": workspace_name,
                "url": "file:{}".format(pathname),
            }
        }

        response = requests.post(
            "{}/workspaces/{}/coveragestores".format(self.uri,
                workspace_name),
            headers=headers,
            data=json.dumps(payload),
            auth=self.auth)

        if response.status_code != 201:
            raise RuntimeError(response.text)


    def delete_coverage_store(self,
            workspace_name,
            name,
            recurse=False):

        headers = {
            "Content-type": "application/json"
        }
        parameters = {
            "recurse": "true" if recurse else "false"
        }
        response = requests.delete(
            "{}/workspaces/{}/coveragestores/{}".format(self.uri,
                workspace_name, name),
            headers=headers,
            params=parameters,
            auth=self.auth)

        if response.status_code != 200:
            raise RuntimeError(response.text)



#!/usr/bin/env python
import codecs
import os
import setuptools
import sys


__version__ = "0.0.4"


here = os.path.abspath(os.path.dirname(__file__))


# Get the long description from the README file
with codecs.open(os.path.join(here, "README.rst"), encoding="utf-8") as file:
    long_description = file.read()


setuptools.setup(
    name="starling",
    version=__version__,
    description="The Starling Python package contains code that might be "
        "useful when implementing services in Python",
    long_description=long_description,
    author="Kor de Jong",
    author_email="k.dejong@geoneric.eu",
    url="https://github.com/geoneric/starling",
    packages=setuptools.find_packages(),  # "source"),
    # package_dir={"": "source"},
    install_requires=[
        "flask>=0.11",
        "tzlocal>=1.3",
    ],
    license="MIT License",
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Topic :: Internet :: WWW/HTTP :: HTTP Servers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="Starling, microservice, service, Flask",
    zip_safe=True,
)



"""
Filters to be used in jinja2 templates
"""
from datetime import datetime
from ..time_point import *


def format_pathname(
        pathname,
        max_length):
    """
    Format a pathname

    :param str pathname: Pathname to format
    :param int max_length: Maximum length of result pathname (> 3)
    :return: Formatted pathname
    :rtype: str
    :raises ValueError: If *max_length* is not larger than 3

    This function formats a pathname so it is not longer than *max_length*
    characters. The resulting pathname is returned. It does so by replacing
    characters at the start of the *pathname* with three dots, if necessary.
    The idea is that the end of the *pathname* is the most important part
    to be able to identify the file.
    """
    if max_length <= 3:
        raise ValueError("max length must be larger than 3")

    if len(pathname) > max_length:
        pathname = "...{}".format(pathname[-(max_length-3):])

    return pathname


def format_time_point(
        time_point_string):

    """
    :param str time_point_string: String representation of a time point
        to format
    :return: Formatted time point
    :rtype: str
    :raises ValueError: If *time_point_string* is not formatted according to
        ISO 8601

    See :py:meth:`datetime.datetime.isoformat` function for supported formats.
    """
    # formats = [
    #     "%Y-%m-%dT%H:%M:%S+HH:MM",
    #     "%Y-%m-%dT%H:%M:%S.%f",
    #     "%Y-%m-%dT%H:%M:%S",
    # ]
    time_point = None

    if len(time_point_string) == 32 and time_point_string[29] == ':':
        # Python cannot parse its own generated isoformat() string...
        # We need to get rid of the semicolon.
        time_point_string = time_point_string[:29] + \
            time_point_string[30:]
        format = "%Y-%m-%dT%H:%M:%S.%f%z"

        try:
            time_point = datetime.strptime(time_point_string, format)
        except:
            pass
    else:

        format = "%Y-%m-%dT%H:%M:%S.%f"

        try:
            time_point = datetime.strptime(time_point_string, format)
        except:
            pass

    # TODO Handle other time formats.

    if time_point is None:
        raise ValueError(
            "time point must be formatted according to ISO 8601 "
            "('{}')".format(time_point_string))

    if not is_aware(time_point):
        time_point = make_aware(time_point)

    time_point = local_time_point(time_point)

    return time_point.strftime("%Y-%m-%dT%H:%M:%S")
